@@ -1,7 +1,10 @@
 #' Create a histogram with outlier bins
 #'
-#' Visualize the distribution of a single continuous variable by dividing the x axis into bins and counting the number of observations in each bin.
-#' Data points that are considered outliers can be binned together. This might be helpful to display numerical data over a very wide range of values in a compact way.
+#' Visualize the distribution of a single continuous variable by dividing the x
+#' axis into bins and counting the number of observations in each bin.
+#' Data points that are considered outliers can be binned together. This might
+#' be helpful to display numerical data over a very wide range of values in a
+#' compact way.
 #'
 #' @param data data.frame
 #' @param x variable name in data.frame `data` that should be mapped
@@ -13,7 +16,8 @@
 #' @param color color for bar lines
 #' @param fill_outliers color used to fill outlier bars
 #'
-#' @details Wrapper function around `ggplot2::geom_histogram()`. The method is based on suggestions from <https://edwinth.github.io/blog/outlier-bin/>.
+#' @details Wrapper function around `ggplot2::geom_histogram()`. The method is
+#' based on suggestions from <https://edwinth.github.io/blog/outlier-bin/>.
 #'
 #' @return a ggplot2 object
 #'
@@ -52,7 +56,8 @@ histbin <- function(data, x, left = NULL, right = NULL, line = FALSE, bins = 30,
 
   if ( isTRUE( line )){
     obj <- obj +
-      ggplot2::stat_density(geom = "line", aes(y = nbinwidth * ..count..), color = "dodgerblue") +
+      ggplot2::stat_density(geom = "line", aes(y = nbinwidth * ..count..),
+                            color = "dodgerblue") +
       ggplot2::ylab("count")
   }
 
@@ -65,12 +70,15 @@ histbin <- function(data, x, left = NULL, right = NULL, line = FALSE, bins = 30,
 
   if ( !is.null(splitsing[[1]]) ){
 
-    ticks_for_left <- update_tickmarks_left(obj, left, round(min(data[[xvar00]], na.rm = TRUE), 1))
+    ticks_for_left <- update_tickmarks_left(
+      obj, left, round(min(data[[xvar00]], na.rm = TRUE), 1)
+      )
 
     obj <- obj +
       ggplot2::geom_histogram(data = splitsing[[1]],
                               fill = fill_outliers,
-                              color = ifelse(fill_outliers == "#a7d1a7", "forestgreen", fill_outliers),
+                              color = ifelse(fill_outliers == "#a7d1a7",
+                                             "forestgreen", fill_outliers),
                               binwidth = nbinwidth) +
       ggplot2::scale_x_continuous(breaks = ticks_for_left$tick_positions,
                                   labels = ticks_for_left$tick_labels)
@@ -78,13 +86,16 @@ histbin <- function(data, x, left = NULL, right = NULL, line = FALSE, bins = 30,
 
   if ( !is.null(splitsing[[3]]) ){
 
-    ticks_for_right <- update_tickmarks_right(obj, right, round(max(data[[xvar00]], na.rm = TRUE), 1))
+    ticks_for_right <- update_tickmarks_right(obj, right,
+                                              round(max(data[[xvar00]],
+                                                        na.rm = TRUE), 1))
 
     suppressMessages(
       obj <- obj +
         ggplot2::geom_histogram(data = splitsing[[3]],
                                 fill = fill_outliers,
-                                color = ifelse(fill_outliers == "#a7d1a7", "forestgreen", fill_outliers),
+                                color = ifelse(fill_outliers == "#a7d1a7",
+                                               "forestgreen", fill_outliers),
                                 binwidth = nbinwidth) +
         ggplot2::scale_x_continuous(breaks = ticks_for_right$tick_positions,
                                     labels = ticks_for_right$tick_labels)
