@@ -110,26 +110,23 @@ update_formula_add <- function(offset_term, fm_no_offset, add_term){
 
 #' @keywords internal
 cut_borders_df <- function(df, col){
-  if (!col %in% names(df)) stop("Column name must be available in data", call. = FALSE)
+  if (!col %in% names(df)) stop("Column name must be available in data",
+                                call. = FALSE)
   df_vec <- df[[col]]
 
   pattern <- "(\\(|\\[)(-*[0-9]+\\.*[0-9]*),(-*[0-9]+\\.*[0-9]*)(\\)|\\])"
   suppressWarnings({
     df$start_oc <- ifelse(gsub(pattern,"\\1", df_vec) == "(", "open",
-                          ifelse(gsub(pattern,"\\1", df_vec) == "[", "closed", NA))
+                          ifelse(gsub(pattern,"\\1", df_vec) == "[",
+                                 "closed", NA))
     df$end_oc <- ifelse(gsub(pattern,"\\4", df_vec) == ")", "open",
-                        ifelse(gsub(pattern,"\\4", df_vec) == "]", "closed", NA))
+                        ifelse(gsub(pattern,"\\4", df_vec) == "]",
+                               "closed", NA))
     df$start_  <- as.numeric(gsub(pattern,"\\2", df_vec))
     df$end_ <- as.numeric(gsub(pattern,"\\3", df_vec))
   })
   df$avg_ <- rowMeans(df[, c('start_', 'end_')], na.rm = TRUE)
 
-  if( any(is.na(df$avg_)) ) stop(
-    "Can't find cut points in column ", col, ". Intervals in ", col,
-    " should be in the same format as the output from cut().
-  Use e.g. cut() or insurancerating::construct_tariff_classes().",
-    call. = FALSE)
-
   return(df)
 }
 
@@ -160,7 +157,8 @@ cut_borders_model <- function(model, x_cut){
 fit_polynomial <- function(borders_model, x_org, degree = NULL, breaks = NULL){
 
   if ( is.null(breaks) ){
-    breaks <- seq(min(borders_model$start_), max(borders_model$end_), length.out = nrow(borders_model))
+    breaks <- seq(min(borders_model$start_), max(borders_model$end_),
+                  length.out = nrow(borders_model))
   }
 
   # Take halfway points of breaks to fit polynomial
@@ -170,7 +168,8 @@ fit_polynomial <- function(borders_model, x_org, degree = NULL, breaks = NULL){
   breaks_mid <- breaks_mid[!is.na(breaks_mid)]
 
   unique_borders <- unique(c(breaks_min, breaks_max))
-  levels_borders <- levels(cut(breaks_min, breaks = unique_borders, include.lowest = TRUE))
+  levels_borders <- levels(cut(breaks_min, breaks = unique_borders,
+                               include.lowest = TRUE, dig.lab = 9))
 
   lm_poly <- lm(estimate ~ poly(avg_, degree = degree), data = borders_model)
   new_poly_df <- data.frame(avg_ = breaks_mid)
@@ -212,11 +211,15 @@ fit_polynomial <- function(borders_model, x_org, degree = NULL, breaks = NULL){
 join_to_nearest <- function(dat, reference, x){
   reference <- data.table::data.table(reference)
   dat <- data.table::data.table(dat)
-  join <- reference[dat, roll = "nearest", on = x, rollends = c(FALSE, FALSE)]
+  reference_nm <- setdiff(names(reference), x)
+  dat <- dat[, c(x) := as.numeric(get(x))]
+  join <- reference[dat, roll = "nearest", on = x][is.na(get(x)),
+                                                   c(reference_nm) := NA]
   as.data.frame(join)
 }
 
 
+
 #' Join restricted data to model data
 #'
 #' @noRd
